//------------------------------------------------------------------
//
// EXPLODED DEMO
//
// A 3D Annotation JavaScript by JJ Ventrella, May, 2005 
//
// This is a concatenation of four separate files:
// Common.js,
// TimeLine.js
// ExplodedView.js
// MouseSelection.js
//
//------------------------------------------------------------------



  //----------------------------
  //----------------------------
  // Common.js
  //----------------------------
  //----------------------------


console.println( "* The Javascript 'Common.js' is now being read..." );



  var ONE_HALF = 0.5;
  var PI       = 3.14159;

  var verbose = false;

  var theBackground = null;

  var backgroundRed   = 0.7;
  var backgroundGreen = 0.7;
  var backgroundBlue  = 0.7;


  //---------------------------------------------------------------------
  function printNamesOfAllTheMeshes()
   {
    var numberOfMeshes = scene.meshes.count;

    console.println( "Printing names of all the meshes in the scene..." );
    console.println( "(there are " + numberOfMeshes + " meshes in this scene)" );

    for (m=0; m<numberOfMeshes; m++)
     {
      var modelMesh = scene.meshes.getByIndex(m);
      console.println( "mesh " + m + " is named '" + modelMesh.name + "'" );
     }


  }//---------------------------------------------------------------------





  //---------------------------------------------------------------------
  function printOutNodeParentalHierarchies()
   {
    console.println( "" );
    console.println( "* printing out the parental hierarchies of all the nodes..." );

     for (m=0; m<scene.nodes.count; m++)
      {
       var indentation = 0;
       var node = scene.nodes.getByIndex( m );
       console.println( "" );
       console.println( "" );
       console.println( "-----------------------------------------" );
       console.print( "Node " + m + " is " );
       traverseParentalTree( node, indentation );
      }

     console.println( "" );

  }//---------------------------------------------------------------------




  //---------------------------------------------------------------------
  function traverseParentalTree( node, indentation )
   {      
    //----------------------------------------------------------     
    // print it  
    //---------------------------------------------------------- 
    var kindOfNode = node.constructor.name;

    console.print( "named '" + node.name + "', and is " );

    if (( kindOfNode == "Scene" )
    ||  ( kindOfNode == "Dummy" )
    ||  ( kindOfNode == "Mesh"  ))
     { 
      console.print( "of type " + kindOfNode ); 
     }
    else 
     { 
      console.print( "an unknown type of node." );
     }

    if ( kindOfNode != "Scene" )
     {
      console.println( node.transform ); 
     }



    //----------------------------------------------------------     
    // recurse
    //----------------------------------------------------------     
    if ( kindOfNode != "Scene" )
     {
      console.println("");

      indentation ++;
      //----------------------------------------------------------     
      // do the indentation   
      //----------------------------------------------------------     
      for (var i=0; i<indentation; i++ ) 
       { 
        console.print( "  " ); 
       }

      console.print( "The parent of '" + node.name + "' is " );

      traverseParentalTree( node.parent, indentation )
     }

  }//---------------------------------------------------------------------




  //---------------------------------------------------------------------
  function setBackgroundColor( r, g, b )
   {      
    backgroundRed   = r;
    backgroundGreen = g;
    backgroundBlue  = b;

    theBackground.setColor(new Color( backgroundRed, backgroundGreen, backgroundBlue ));
   }




  //------------------------------------------------------------------------------
  //------------------------------------------------------------------------------
  // Creating a Background using the Render Event Handler
  //------------------------------------------------------------------------------
  //------------------------------------------------------------------------------
  renderEventHandler = new RenderEventHandler();
  renderEventHandler.onEvent = function( event )
   {
    runtime.removeEventHandler( this );
    theBackground = event.canvas.background;

  //theBackground.setColor( new Color( backgroundRed, backgroundGreen, backgroundBlue ) );
   }

  runtime.addEventHandler( renderEventHandler );


console.println( "* OK, the Javascript 'Common.js' was successfully read to the end." );





//----------------------------
//----------------------------
// TimeLine.js
//----------------------------
//----------------------------

console.println( "* The Javascript 'TimeLine.js' is now being read..." );


  //---------------------------------------------
  //---------------------------------------------
  // Time Line Object Constructor
  //---------------------------------------------
  //---------------------------------------------
  function TimeLineObject()
   {

timeLineOwner = this;

    //----------------------------------
    // private members
    //----------------------------------
    var a = -1;
    var TIME_LINE_STAGE_STARTING    = a; a++;
    var TIME_LINE_STAGE_RUNNING     = a; a++;
    var TIME_LINE_STAGE_ENDING      = a; a++;
    var TIME_LINE_STAGE_NOT_RUNNING = a; a++;

    var stage         = TIME_LINE_STAGE_NOT_RUNNING;
    var running       = false;
    var timePassed    = 0.0;
    var duration      = 0.0;



    //---------------------------------------------
    // public methods
    //---------------------------------------------
    this.start = function( d )
     {
      stage         	= TIME_LINE_STAGE_STARTING;
      running       	= true;
      timePassed    	= 0.0;
      duration      	= d;
     }

    //---------------------------------
    this.stop = function()
     {
      stage         	= TIME_LINE_STAGE_NOT_RUNNING;
      running       	= false;
     }

    //---------------------------------
    this.update = function( deltaTime )
     {
      if ( running )
       {
        timePassed += deltaTime;

        if ( stage == TIME_LINE_STAGE_ENDING )
         {
          stage = TIME_LINE_STAGE_NOT_RUNNING;
          running = false;
         }
        else if ( timePassed > duration ) // this allows one more step to happen to make sure smoothed value ends at 1.0;
         {
          stage = TIME_LINE_STAGE_ENDING;
         } 
       }
     }


    //-----------------------------------------------------------------------
    this.getValue = function() 
     { 
      if ( duration == 0.0 )
       {
        return 1.0;
       }
      else
       {
        var value = timePassed / duration;
  
        if ( value > 1.0 )
         {
          return 1.0;
         }
        else
         {
          return value; 
         }
       }
     }

    this.getEaseInOutValue = function()
     { 
      var value = timeLineOwner.getValue();
      return ONE_HALF - ONE_HALF * Math.cos( value * Math.PI ); 
     }

    this.getEaseInValue = function()
     { 
      var value = timeLineOwner.getValue();
      return value * value; 
     }

    this.getEaseOutValue = function()
     { 
      var value = timeLineOwner.getValue();
      return Math.sqrt( value ); 
     }


    this.getRunning       = function() { return running;       }
    this.getTimePassed    = function() { return timePassed;    }

   }


console.println( "* OK, the Javascript 'TimeLine.js' was successfully read to the end." );




//----------------------------
//----------------------------
// ExplodedView.js
//----------------------------
//----------------------------

console.println( "* The Javascript 'ExplodedView.js' is now being read..." );


  //---------------------------------------------
  //---------------------------------------------
  // ExplodedView Object Constructor
  //---------------------------------------------
  //---------------------------------------------
  function ExplodedViewObject()
   {
    explodedViewObject = this;

    //----------------------------------
    // private members
    //----------------------------------
    var a = 0;
    var LINEAR_ANIMATION_RATE      = a; a++;
    var EASE_IN_ANIMATION_RATE     = a; a++;
    var EASE_OUT_ANIMATION_RATE    = a; a++;
    var EASE_IN_OUT_ANIMATION_RATE = a; a++;
    var SPRINGY_ANIMATION_RATE 	   = a;  

    var INITIAL_SPRINGY_RANDOMNESS = 0.2;
    var SPRING_FORCE               = 0.5;
    var SPRING_FRICTION            = 0.95;

    var animationTimeLine 	= new TimeLineObject();
    var numParts 		= scene.meshes.count;
    var defaultPosition 	= new Array( numParts );
    var explodedPosition 	= new Array( numParts );
    var interpolatedPosition	= new Array( numParts );
    var velocity		= new Array( numParts ); // used only in springy mode

    var timeEventHandler 	= new TimeEventHandler();
    var duration    		= 1.0;
    var animationRate 		= EASE_IN_OUT_ANIMATION_RATE;
    var currentlyCycling 	= false;
    var cyclingMode	 	= false;


    for (var p=0; p<numParts; p++)
     {
      var mesh = scene.meshes.getByIndex(p);

      defaultPosition	  [p] = new Vector3();
      explodedPosition	  [p] = new Vector3();
      interpolatedPosition[p] = new Vector3();
      velocity            [p] = new Vector3();
      defaultPosition	  [p].set( mesh.transform.translation );
      explodedPosition	  [p].set( mesh.transform.translation ); // to be re-configured at some point
      interpolatedPosition[p].set( mesh.transform.translation ); // dynamic during animation
      velocity            [p].set3( 0.0, 0.0, 0.0 ); 
     }


    //------------------------------------
    // configure the Time Event Handler
    //------------------------------------
    timeEventHandler.onTimeChange = true;
    timeEventHandler.onEvent = function( event )
     {
      if ( cyclingMode )
       {
        if ( currentlyCycling )
         {
          if ( ! animationTimeLine.getRunning() )
           {
            explodedViewObject.startAnimation();
           }
         }
       }


      if ( animationTimeLine.getRunning() )
       {
        animationTimeLine.update( event.deltaTime );

        var interpolationFraction = 0.0;

        if ( animationRate == EASE_IN_ANIMATION_RATE )
         {
          interpolationFraction = animationTimeLine.getEaseInValue();
         }
        else if ( animationRate == EASE_OUT_ANIMATION_RATE )
         {
          interpolationFraction = animationTimeLine.getEaseOutValue();
         }
        else if ( animationRate == EASE_IN_OUT_ANIMATION_RATE )
         {
          interpolationFraction = animationTimeLine.getEaseInOutValue();
         }
        else 
         {
          interpolationFraction = animationTimeLine.getValue();
         }


        for (var p=0; p<scene.meshes.count; p++)
         {
          var mesh = scene.meshes.getByIndex( p );

          var shiftVector = new Vector3();

          shiftVector.set( explodedPosition[p] );
          shiftVector.subtractInPlace( defaultPosition[p] );

          if ( animationRate == SPRINGY_ANIMATION_RATE )
           {
            var v = new Vector3();
            v.set( explodedPosition[p] );
            v.subtractInPlace( interpolatedPosition[p] );
            var force = SPRING_FORCE / ( 1.0 + duration );
            velocity[p].addScaledInPlace( v, force * event.deltaTime );
            velocity[p].scaleInPlace( SPRING_FRICTION );
            interpolatedPosition[p].addInPlace( velocity[p] );
            // the following line ensures that interpolatedPosition always ends up in the right place...
            interpolatedPosition[p].addScaledInPlace( v, animationTimeLine.getEaseInValue() );
           }
          else
           {
            interpolatedPosition[p].set( defaultPosition[p] );
            interpolatedPosition[p].addScaledInPlace( shiftVector, interpolationFraction );
           }

          mesh.transform.setIdentity();
          mesh.transform.translateInPlace( interpolatedPosition[p] );

         }// for loop

       }//if ( animationTimeLine.getRunning() )

     }//timeEventHandler.onEvent function

    runtime.addEventHandler( timeEventHandler );


    //---------------------------------------------
    // public methods
    //---------------------------------------------
    this.saveAllMeshesAsDefaultConfiguration = function()
     {
      for (var p=0; p<scene.meshes.count; p++)
       {
        var mesh = scene.meshes.getByIndex( p );
        defaultPosition[p].set( mesh.transform.translation );
       }
     }


    this.saveAllMeshesAsExplodedConfiguration = function()
     {
      for (var p=0; p<scene.meshes.count; p++)
       {
        var mesh = scene.meshes.getByIndex( p );
        explodedPosition[p].set( mesh.transform.translation );
       }
      currentlyCycling = false;  
     }


    this.setToDefaultConfiguration = function()
     {
      animationTimeLine.stop();

      for (var p=0; p<scene.meshes.count; p++)
       {
        var mesh = scene.meshes.getByIndex( p );
        mesh.transform.setIdentity();
        mesh.transform.translateInPlace( defaultPosition[p] ); 
        interpolatedPosition[p].set( defaultPosition[p] );
       }
      currentlyCycling = false;  
     }


    this.setToExplodedConfiguration = function()
     {
      animationTimeLine.stop();

      for (var p=0; p<scene.meshes.count; p++)
       {
        var mesh = scene.meshes.getByIndex( p );
        mesh.transform.setIdentity();
        mesh.transform.translateInPlace( explodedPosition[p] ); 
        interpolatedPosition[p].set( explodedPosition[p] );
       }
      currentlyCycling = false;  
     }


    this.setDuration = function( d )
     {
      duration = d;

      if (( cyclingMode ) 
      &&  ( currentlyCycling ))
       {
        explodedViewObject.startAnimation();
       }
     }


    this.setAnimationRateToLinear = function()
     {
      animationRate = LINEAR_ANIMATION_RATE;
      explodedViewObject.setToDefaultConfiguration();
     }
    this.setAnimationRateToEaseIn = function()
     {
      animationRate = EASE_IN_ANIMATION_RATE;
      explodedViewObject.setToDefaultConfiguration();
     }
    this.setAnimationRateToEaseOut = function()
     {
      animationRate = EASE_OUT_ANIMATION_RATE;
      explodedViewObject.setToDefaultConfiguration();
     }
    this.setAnimationRateToEaseInOut = function()
     {
      animationRate = EASE_IN_OUT_ANIMATION_RATE;
      explodedViewObject.setToDefaultConfiguration();
     }
    this.setAnimationRateToSpringy = function()
     {
      animationRate = SPRINGY_ANIMATION_RATE;
      explodedViewObject.setToDefaultConfiguration();
     }


    this.setCycleToOnce = function()
     {
      cyclingMode = false;
      currentlyCycling = false;
     }

    this.setCycleToRepeating = function()
     {
      cyclingMode = true;
     }



    this.startAnimation = function()
     {
      if ( animationRate == SPRINGY_ANIMATION_RATE )
       {
        for (var p=0; p<scene.meshes.count; p++)
         {
          interpolatedPosition[p].set( defaultPosition[p] );
          velocity[p].set3
           ( 
            -INITIAL_SPRINGY_RANDOMNESS * ONE_HALF + INITIAL_SPRINGY_RANDOMNESS * Math.random(),
            -INITIAL_SPRINGY_RANDOMNESS * ONE_HALF + INITIAL_SPRINGY_RANDOMNESS * Math.random(),
            -INITIAL_SPRINGY_RANDOMNESS * ONE_HALF + INITIAL_SPRINGY_RANDOMNESS * Math.random()
           );
         }
       }

      if ( cyclingMode )
       {
        currentlyCycling = true;
       }

      animationTimeLine.start( duration );
     }



    //-----------------------------------------------------------------------
    // save the current mesh configuration as the default. 
    // This will be the initial state of the model before exploding. 
    //-----------------------------------------------------------------------
    this.saveAllMeshesAsDefaultConfiguration();


   } //------ end of ExplodedView Object Constructor ---------------------




  //------------------------------------------------------------------------
  // Here is the public method in which a new ExplodedViewObject is
  // created, so that the calls can be made 
  //------------------------------------------------------------------------
  function createNewExplodedViewObject()
   {
    return new ExplodedViewObject();

   }//--------------------------------------------------------------------------


console.println( "* OK, the Javascript 'ExplodedView.js' was successfully read to the end." );


//----------------------------
//----------------------------
// MouseSelection.js
//----------------------------
//----------------------------

console.println( "* The Javascript 'MouseSelection.js' is now being read..." );


  //---------------------------------------------
  //---------------------------------------------
  // model part Object Constructor
  //---------------------------------------------
  //---------------------------------------------
  function ModelPartObject()
   {
    this.defaultTransform 	= new Matrix4x4();
    this.adjustedTransform 	= new Matrix4x4();
    this.mouseEventHandler 	= null;
    this.mesh              	= null;
   }


  //---------------------------------------------
  //---------------------------------------------
  // MouseSelection Object Constructor
  //---------------------------------------------
  //---------------------------------------------
  function MouseSelectionObject()
   {
    owner = this;

    //-------------------------------------------
    //  variables used as constants
    //-------------------------------------------
    var DEFAULT_RENDER_MODE            	= "default";
    var DEFAULT_MOUSE_OVER_RENDER_MODE  = "shaded illustration";
    var DEFAULT_MOUSE_DOWN_RENDER_MODE  = "illustration";
    var DEFAULT_SELECTED_RENDER_MODE	= "shaded illustration";


    //-------------------------------------------------------------
    //-------------------------------------------------------------
    // private members of MouseSelection Object
    //-------------------------------------------------------------
    //-------------------------------------------------------------
    var defaultRenderMode		= DEFAULT_RENDER_MODE;
    var mouseOverRenderMode 		= DEFAULT_MOUSE_OVER_RENDER_MODE;
    var mouseDownRenderMode   		= DEFAULT_MOUSE_DOWN_RENDER_MODE;
    var selectedRenderMode    		= DEFAULT_SELECTED_RENDER_MODE;
    var mouseSelectionMessages		= false;
    var mousePart			= -1;
    var selectedPart			= -1;
    var initialized			= false;
    var numberOfModelParts		= 0;
    var atLeastOnePartHasBeenCreated	= false;
    var mouseDragDistance 		= 0.0;
    var mouseDragOffset 		= new Vector3();
    var cameraPositionWhileDraggingPart = new Vector3();
    var cameraTargetWhileDraggingPart   = new Vector3();
    var cameraFOVWhileDraggingPart 	= 0.0;
    var camera 				= scene.cameras.getByIndex( 0 );
    var draggedPart 			= -1;
    var draggingEnabled                 = false;
    var generalMouseEventHandler 	= new MouseEventHandler();
    var mouse_X 			= 0;
    var mouse_Y 			= 0;
    var canvasWidth  			= 0;
    var canvasHeight 			= 0;


    //--------------------------------------
    // making an array of model parts
    //--------------------------------------
    var maxNumModelParts = scene.meshes.count;
    var modelPart = new Array( maxNumModelParts );

    for (p=0; p<maxNumModelParts; p++)
     {
      modelPart[p] = new ModelPartObject();
     }






   //--------------------------------------------------
   //--------------------------------------------------
   // public methods
   //--------------------------------------------------
   //--------------------------------------------------








  //------------------------------------------------------
  // General Mouse Event Handler for whole 3D Annotation
  //------------------------------------------------------
  generalMouseEventHandler.onMouseDown	= true;
  generalMouseEventHandler.onMouseMove	= true;
  generalMouseEventHandler.onMouseUp	= true;
  generalMouseEventHandler.onEvent 	= function( event )
   {
    mouse_X = event.mouseX;
    mouse_Y = event.mouseY;

    canvasWidth  = event.canvasPixelWidth;
    canvasHeight = event.canvasPixelHeight;

    //-----------------------------------------------
    // mouse is moving
    //-----------------------------------------------
    if ( draggingEnabled )
     {
      if ( event.isMouseMove )
       {
        if ( draggedPart != -1 )
         {
          owner.dragPart();
         }
       }
     }

    //-----------------------------------------------
    // mouse just up
    //-----------------------------------------------
    if ( event.isMouseUp )
     {
      owner.saveAllPartsAsAdjustedTransforms();

      if ( mousePart != -1 )
       {
        modelPart[ mousePart ].mesh.renderMode = defaultRenderMode;
        mousePart = -1;
       }

      draggedPart = -1;
     }

   } // generalMouseEventHandler.onEvent method


  runtime.addEventHandler( generalMouseEventHandler );








   //--------------------------------------------------------------------------------
   // function used by the Mouse Event Handler for each individual part (mesh)
   //--------------------------------------------------------------------------------
   this.myMouseHandlingFunction = function( event ) 
    {
     //-------------------------------------------------------------
     // Let's find the mesh index for this mouse event handler
     //-------------------------------------------------------------
     var m = 0;
     var lookingForMesh = true;
     while ( lookingForMesh )
      {
       if ( this.target == modelPart[m].mesh )
        {
         lookingForMesh = false;

         //-----------------------------------------------
         // mouse just over
         //-----------------------------------------------
         if ( event.isMouseOver )
          {
           if ( ! event.leftButtonDown )
            {
             mousePart = m;
             modelPart[ m ].mesh.renderMode = mouseOverRenderMode;
            }
          }

         //-----------------------------------------------
         // mouse just out
         //-----------------------------------------------
         if ( event.isMouseOut )
          {
           if (( ! event.leftButtonDown )
           &&  ( m != selectedPart ))
            {
             mousePart = -1;
             modelPart[ m ].mesh.renderMode = defaultRenderMode;
            }
          }


         //-----------------------------------------------
         // mouse just down
         //-----------------------------------------------
         if ( event.isMouseDown )
          {
           mousePart = m;
           draggedPart = m;
           modelPart[ m ].mesh.renderMode = mouseDownRenderMode;

           var cameraToObjectVector = new Vector3();
           cameraToObjectVector.set( modelPart[ mousePart ].mesh.transform.translation );
           cameraToObjectVector.subtractInPlace( camera.position );
           mouseDragDistance = cameraToObjectVector.length;

           var mousePointingVector = new Vector3();
           mousePointingVector.set( camera.getDirectionFromScreen( event.mouseX, event.mouseY, event.canvasPixelWidth, event.canvasPixelHeight ) );
           mousePointingVector.scaleInPlace( mouseDragDistance );

           var mousePointingPosition = new Vector3();
           mousePointingPosition.set( camera.position );
           mousePointingPosition.addInPlace( mousePointingVector );

           mouseDragOffset.set( modelPart[ mousePart ].mesh.transform.translation );
           mouseDragOffset.subtractInPlace( mousePointingPosition );

           cameraPositionWhileDraggingPart.set( camera.position );
           cameraTargetWhileDraggingPart.set( camera.targetPosition );
           cameraFOVWhileDraggingPart = camera.fov;
          }


         //-----------------------------------------------
         // mouse just up
         //-----------------------------------------------
         if ( event.isMouseUp )
          {
           owner.saveAllPartsAsAdjustedTransforms();

           if ( selectedPart != -1 )
            {
             modelPart[ selectedPart ].mesh.renderMode = defaultRenderMode; 
            }

           selectedPart = m;
           modelPart[ m ].mesh.renderMode = selectedRenderMode;
           mousePart = -1;
          }



        }//if ( this.target == modelPart[m].mesh )

       m ++;

       if ( m > numberOfModelParts - 1 )
        {
         lookingForMesh = false;
        }

      }// while loop 

    }//------- myMouseHandlingFunction -------------------------------



   //-------------------------------------------
   // drag the part
   //-------------------------------------------
   this.dragPart = function() 
    {
     if ( mousePart != -1 )
      {
       camera.position.set      ( cameraPositionWhileDraggingPart );
       camera.targetPosition.set( cameraTargetWhileDraggingPart   );
       camera.fov =               cameraFOVWhileDraggingPart;

       var mousePointingVector = new Vector3();
       mousePointingVector.set( camera.getDirectionFromScreen( mouse_X, mouse_Y, canvasWidth, canvasHeight ) );
       mousePointingVector.scaleInPlace( mouseDragDistance );
      
       var mousePointingPosition = new Vector3();
       mousePointingPosition.set( camera.position );
       mousePointingPosition.addInPlace( mousePointingVector );

       var mouseDragPosition = new Vector3();
       mouseDragPosition.set( mousePointingPosition );
       mouseDragPosition.addInPlace( mouseDragOffset );
          
       //console.println( "mouse down part position = " + modelPart[ mousePart ].mesh.transform.translation );
       //console.println( "camera position          = " + camera.position );
       //console.println( "mouse down part distance = " + mouseDragDistance );
       //console.println( "mouse pointing vector    = " + mousePointingVector );
       //console.println( "mouse drag position      = " + mouseDragPosition );
       //console.println( "" );          

       modelPart[ mousePart ].mesh.transform.setIdentity();
       modelPart[ mousePart ].mesh.transform.translateInPlace( mouseDragPosition );
      }

    }//--------------------------------------------------------------------------






  //---------------------------------------------------------------------
  this.getMeshesFromModel = function()
   {
    numberOfModelParts = 0;

    if ( mouseSelectionMessages ) console.println( "" );
    if ( mouseSelectionMessages ) console.println( "* including all (material) meshes...................." );

     for (p=0; p<scene.meshes.count; p++)
      {
       var modelMesh = scene.meshes.getByIndex( p );

       if ( ! modelMesh.material ) // it aint got no Material, ya'll!
        { 
         if ( mouseSelectionMessages ) console.println( "" );
         if ( mouseSelectionMessages ) console.println( "* Yo: mesh " + p + " has no material, so I am not going to include it." );
        }
       else
        {
         if ( mouseSelectionMessages ) 
          {
           console.println( "" );
           console.println( "* including mesh " + p + " called '" + modelMesh.name + "' as part " + numberOfModelParts );
          }

         modelPart[ numberOfModelParts ].mesh = modelMesh;
         modelPart[ numberOfModelParts ].defaultTransform.set( modelPart[ numberOfModelParts ].mesh.transform );


         if ( ! atLeastOnePartHasBeenCreated )
          {
           atLeastOnePartHasBeenCreated = true;
          }


         numberOfModelParts ++;
        }
      }

  }//---------------------------------------------------------------------




//---------------------------------------------------------------------
this.update = function( deltaTime )
 {
 }//---------------------------------------------------------------------







//---------------------------------------------------------------------
this.initialize = function()
 {
  if ( ! initialized )
  {
   //--------------------------------------
   // Set up a Time Event Handler
   //--------------------------------------
   timeEventHandler = new TimeEventHandler();
   timeEventHandler.onTimeChange = true;
   timeEventHandler.onEvent = function( event )
    {
     owner.update( event.deltaTime );
    }
   runtime.addEventHandler( timeEventHandler );



   owner.getMeshesFromModel();

   //scene.defaultRenderOptions.shadedIllustrationRenderModeLineColor.set( 1.0, 1.0, 1.0 );
   //scene.defaultRenderOptions.illustrationRenderModeLineColor.set      ( 1.0, 1.0, 1.0 );
   //scene.defaultRenderOptions.solidRenderModeLineColor.set             ( 1.0, 1.0, 1.0 );

   if ( mouseSelectionMessages ) 
    {
     console.println( "" );
     console.println( "* 'initializing Mouse Selection JavaScript'" );


     if ( atLeastOnePartHasBeenCreated )
      {
       console.println( "* There are " + numberOfModelParts + " parts" );
      }
     else
      {
       console.println( "* NO parts have been created" );
      }

    }// if mouseSelectionMessages


     //------------------------------------------------
     // create the mouse event handler array objects, 
     // and add them to the runtime engine
     //------------------------------------------------
     for (var m=0; m<numberOfModelParts; m++)
      {
       modelPart[m].mouseEventHandler = new MouseEventHandler();
       modelPart[m].mouseEventHandler.onMouseMove = true;
       modelPart[m].mouseEventHandler.onMouseDown = true;
       modelPart[m].mouseEventHandler.onMouseUp   = true;
       modelPart[m].mouseEventHandler.onMouseOver = true;
       modelPart[m].mouseEventHandler.onMouseOut  = true;
       modelPart[m].mouseEventHandler.target      = modelPart[m].mesh;
       modelPart[m].mouseEventHandler.onEvent     = owner.myMouseHandlingFunction; // note: not a function call, but a reference to a function
       runtime.addEventHandler( modelPart[m].mouseEventHandler );
      }

     //------------------------------------------------
     // set the default render modes
     //------------------------------------------------
     for (p=0; p<scene.meshes.count; p++)
      {
       var mesh = scene.meshes.getByIndex(p);
       if ( mesh.material ) 
        {

         mesh.renderMode = defaultRenderMode;
        }
      }




  initialized = true;

  } // if not initialized


 }//---------------------------------------------------------------------


  //---------------------------------------------------------------------
  this.setAllPartsToDefaultRenderMode = function()
   {
     for (var m=0; m<numberOfModelParts; m++)
      {
       modelPart[m].mesh.renderMode = defaultRenderMode;
      }

   }//--------------------------------------------------------------------------

  //------------------------------------------------------
  this.selectPart = function( partName )
   {
    owner.selectPartByIndex( getPartIndexFromName( partName ) );

   }//--------------------------------------------------------------------------

  //------------------------------------------------------
  this.unSelectPart = function( partName )
   {
    owner.unSelectPartByIndex( getPartIndexFromName( partName ) );

   }//--------------------------------------------------------------------------




  //------------------------------------------------------
  this.selectPartByIndex = function( partIndex )
   {
    selectedPart = partIndex;

    modelPart[ selectedPart ].mesh.renderMode = selectedRenderMode;

    if ( mouseSelectionMessages ) console.println( "" );
    if ( mouseSelectionMessages ) console.println( "* '" + modelPart[ selectedPart ].mesh.name + "' is now selected" );

   }//--------------------------------------------------------------------------


  //------------------------------------------------------
  this.unSelectPartByIndex = function( partIndex )
   {
    if ( partIndex == selectedPart )
     {
      if ( mouseSelectionMessages ) console.println( "" );
      if ( mouseSelectionMessages ) console.println( "* UN-selecting '" + modelPart[ selectedPart ].mesh.name );

      modelPart[ selectedPart ].mesh.renderMode = defaultRenderMode;
      selectedPart = -1;
    }

   }//--------------------------------------------------------------------------




  //------------------------------------------------------
  this.incrementSelectedPart = function()
   {
    if ( selectedPart == -1 )
     {
      owner.selectPartByIndex( 0 );
     }
    else
     {
      var previouslySelectedPart = selectedPart;

      owner.unSelectPartByIndex( selectedPart );

      var nextPart = previouslySelectedPart + 1;

      if ( nextPart >= numberOfModelParts )
       {
        nextPart = 0;
       }

      owner.selectPartByIndex( nextPart );
    }

   }//--------------------------------------------------------------------------



  //------------------------------------------------------
  this.decrementSelectedPart = function()
   {
    if ( selectedPart == -1 )
     {
      owner.selectPartByIndex( 0 );
     }
    else
     {
      var previouslySelectedPart = selectedPart;

      owner.unSelectPartByIndex( selectedPart );

      var nextPart = previouslySelectedPart - 1;

      if ( nextPart < 0 )
       {
        nextPart = numberOfModelParts - 1;
       }

      owner.selectPartByIndex( nextPart );
    }

   }//--------------------------------------------------------------------------




  //------------------------------------------------------
  this.getSelectedPartName = function()
   {
    result = "there is no selected part";

    if ( selectedPart != -1 )
     {
      result = modelPart[ selectedPart ].mesh.name; 
     }

    return result;

   }//---------------------------------------------------


  //------------------------------------------------------
  this.unSelectSelectedPart = function()
   {
    owner.unSelectPartByIndex( selectedPart );

   }//---------------------------------------------------




    //-------------------------------------------------------------------------------------------
    this.turnOnMessagesInTheConsole = function()
     {
      mouseSelectionMessages = true;

     }//--------------------------------------------------------------------------
    
    //-------------------------------------------------------------------------------------------
    this.turnOffMessagesInTheConsole = function()
     {
      mouseSelectionMessages = false;

     }//--------------------------------------------------------------------------



  //---------------------------------------------------------------------
  this.setDefaultRenderMode = function( renderMode )
   {
    defaultRenderMode = renderMode;
    owner.setAllPartsToDefaultRenderMode();

   }//--------------------------------------------------------------------------

  //---------------------------------------------------------------------
  this.setMouseOverRenderMode = function( renderMode )
   {
    mouseOverRenderMode = renderMode;

   }//--------------------------------------------------------------------------

  //---------------------------------------------------------------------
  this.setMouseDownRenderMode = function( renderMode )
   {
    mouseDownRenderMode = renderMode;

   }//--------------------------------------------------------------------------

  //---------------------------------------------------------------------
  this.setSelectedRenderMode = function( renderMode )
   {
    selectedRenderMode = renderMode;

   }//--------------------------------------------------------------------------

  //---------------------------------------------------------------------
  this.setAllMouseSelectionPartsToDefaultRenderMode = function()
   {
    setAllPartsToDefaultRenderMode();

   }//--------------------------------------------------------------------------


  //-------------------------------------------------------------------
  this.getPartIndexFromName = function ( partName )
   {
    var index = -1;

    for (m=0; m<numberOfModelParts; m++)
     {
      if ( partName == modelPart[m].mesh.name )
       {
        index = m;
       }
     }

   return index;

   }//---------------------------------------------------------------------




  //---------------------------------------------------------------------
  this.setAllPartsToDefaultTransforms = function()
   {
     for (var m=0; m<numberOfModelParts; m++)
      {
       modelPart[m].mesh.transform.set( modelPart[m].defaultTransform );
      }

   }//--------------------------------------------------------------------------



  //---------------------------------------------------------------------
  this.setAllPartsToAdjustedTransforms = function()
   {
     for (var m=0; m<numberOfModelParts; m++)
      {
       modelPart[m].mesh.transform.set( modelPart[m].adjustedTransform );
      }

   }//--------------------------------------------------------------------------



  //---------------------------------------------------------------------
  this.saveAllPartsAsAdjustedTransforms = function()
   {
     for (var m=0; m<numberOfModelParts; m++)
      {
       modelPart[m].adjustedTransform.set( modelPart[m].mesh.transform );
      }

   }//--------------------------------------------------------------------------


  //---------------------------------------------------------------------
  this.setDraggingEnabled = function()
   {
    draggingEnabled = true;

   }//--------------------------------------------------------------------------



  //---------------------------------------------------------------------
  this.setDraggingDisabled = function()
   {
    draggingEnabled = false;

   }//--------------------------------------------------------------------------


    //------------------------------------------------------------------
    // IMPORTANT! - don't forget to call this.
    //------------------------------------------------------------------
    owner.initialize();


   }//------------------------------------------------------------------
   // 
  //  End of MouseSelection Object Constructor
 // 
//------------------------------------------------------------------





  //------------------------------------------------------------------------
  // Here is the public method in which a new mouse selection object is
  // created, so that the calls can be made 
  //------------------------------------------------------------------------
  function createNewMouseSelectionObject()
   {
    return new MouseSelectionObject();

   }//--------------------------------------------------------------------------

console.println( "* OK, the Javascript 'MouseSelection.js' was successfully read to the end." );


